"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_codegen_visitor_plugin_common_1 = require("graphql-codegen-visitor-plugin-common");
var index_1 = require("./index");
var autoBind = require("auto-bind");
var graphql_codegen_core_1 = require("graphql-codegen-core");
var changeCase = require("change-case");
var StencilApolloVisitor = /** @class */ (function (_super) {
    __extends(StencilApolloVisitor, _super);
    function StencilApolloVisitor(fragments, rawConfig) {
        var _this = _super.call(this, fragments, rawConfig, {
            componentType: graphql_codegen_visitor_plugin_common_1.getConfigValue(rawConfig.componentType, index_1.StencilComponentType.functional)
        }) || this;
        autoBind(_this);
        return _this;
    }
    StencilApolloVisitor.prototype.getImports = function () {
        var baseImports = _super.prototype.getImports.call(this);
        var imports = [];
        if (this.config.componentType === index_1.StencilComponentType.class) {
            imports.push("import { Component } from '@stencil/core';");
        }
        return [baseImports].concat(imports).join('\n');
    };
    StencilApolloVisitor.prototype._buildOperationFunctionalComponent = function (node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        var operationName = this.convertName(node.name.value);
        var propsTypeName = this.convertName(operationName + 'Props');
        var apolloStencilComponentTag = changeCase.paramCase("Apollo" + operationType);
        var onReadySignature = graphql_codegen_core_1.toPascalCase("On" + operationType + "ReadyFn");
        var componentName = this.convertName(operationName + "Component");
        var propsVar = "\n        export type " + propsTypeName + " = {\n            variables ?: " + operationVariablesTypes + ";\n            onReady ?: import('stencil-apollo/dist/types/components/" + apolloStencilComponentTag + "/types')." + onReadySignature + "<" + operationResultType + ", " + operationVariablesTypes + ">;\n        };\n      ";
        var component = "\n        export const " + componentName + " = (props: " + propsTypeName + ") => <" + apolloStencilComponentTag + " " + operationType.toLowerCase() + "={ " + documentVariableName + " } { ...props } />;\n      ";
        return [propsVar, component].filter(function (a) { return a; }).join('\n');
    };
    StencilApolloVisitor.prototype._buildClassComponent = function (node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        var componentName = this.convertName(node.name.value + 'Component');
        var apolloStencilComponentTag = changeCase.paramCase("Apollo" + operationType);
        var onReadySignature = graphql_codegen_core_1.toPascalCase("On" + operationType + "ReadyFn");
        return "\n            @Component({\n                tag: '" + changeCase.paramCase("Apollo" + graphql_codegen_core_1.toPascalCase(node.name.value)) + "'\n            })\n            export class " + componentName + " {\n                @Prop() onReady: import('stencil-apollo/dist/types/components/" + apolloStencilComponentTag + "/types')." + onReadySignature + "<" + operationResultType + ", " + operationVariablesTypes + ">;\n                render() {\n                    return <" + apolloStencilComponentTag + " " + operationType.toLowerCase() + "={ " + documentVariableName + " } onReady={ this.onReady } />;\n                }\n            }\n      ";
    };
    StencilApolloVisitor.prototype.buildOperation = function (node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        switch (this.config.componentType) {
            case index_1.StencilComponentType.class:
                return this._buildClassComponent(node, documentVariableName, operationType, operationResultType, operationVariablesTypes);
            case index_1.StencilComponentType.functional:
                return this._buildOperationFunctionalComponent(node, documentVariableName, operationType, operationResultType, operationVariablesTypes);
            default:
                return '';
        }
    };
    return StencilApolloVisitor;
}(graphql_codegen_visitor_plugin_common_1.ClientSideBaseVisitor));
exports.StencilApolloVisitor = StencilApolloVisitor;
//# sourceMappingURL=visitor.js.map